#include "config.h"
#include <sys/time.h>
#include <signal.h>
#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <XVRemoteWindowX.h>
#include "StereoVidere.h"

//#define REMOVE_FLOOR
#define SHOW_DISPLAY

#ifdef REMOVE_FLOOR
#include <XVBlobs.h>
#endif

typedef StereoVidere<XVImageRGB<XV_RGB>, XVImageScalar<u_short> > VID;
typedef XVRemoteWindowX<XV_RGB>                                     WIN;

static VID* vid;

void sighndl(int ws) {
  cerr << "SIGINT called" << endl;
  if(vid) delete vid;
  exit(0);
}

void sighndl2(int ws) {
  cerr << "SIGSEGV called" << endl;
  if(vid) delete vid;
  exit(0);
}

int main(int argc, char* argv[]) {
  signal(SIGINT, sighndl);
  signal(SIGSEGV, sighndl2);

  char * config = getenv("VIDERE_CONFIG");

  if(config) {
    vid = new VID(config);
  }
  else {
    vid = new VID();
  }
  
#ifdef SHOW_DISPLAY
  XVImageScalar<u_short> *disp_image, *win_image;
  XVImageRGB<XV_RGB>     im(320,240);
  WIN 			 win(im);
  win.map();
#endif

  vid->wait_for_completion(0);
  vid->wait_for_completion(0);
  vid->wait_for_completion(0);

  struct timeval start_time, end_time;

  while(1) {
    gettimeofday(&start_time, NULL);
    
    for(int i = 0; i < 50; i++) {
      vid->wait_for_completion(0);

      disp_image = vid->calc_stereo();
      if(!disp_image) continue;
#ifdef REMOVE_FLOOR
      XVRectangleBlob roi(150, 200, 10, 10);
      CALIBRATION_PARAMS params = vid->calibrate(*disp_image);
      XVImageScalar<u_short> roiIM = subimage(*disp_image, roi);
      vid->removeFloor(params, roiIM, roiIM.PosY());
#endif
      win_image = vid->calc_display();

      ScalartoRGB(*win_image, im);
      win.CopySubImage(im);

#ifdef SHOW_DISPLAY
      win.swap_buffers();
      win.flush();
#endif
    }

    gettimeofday(&end_time, NULL);

    cout << 50 / ((end_time.tv_sec - start_time.tv_sec) + (end_time.tv_usec - start_time.tv_usec) * 1e-6)
	 << " [Hz]"<< endl;
  }

  delete vid;
  return 1;
}

