// *** BEGIN_XVISION2_COPYRIGHT_NOTICE ***
// *** END_XVISION2_COPYRIGHT_NOTICE ***

#include <XVColorImage.h>
#include <XVPixel.h>
#include <XVImageRGB.h>
#include <XVImageYUV.h>
#include <XVImageHSV.h>
#include <XVImageScalar.h>

#define CONVERT_OP_IMPL(IM_TYPE) \
template <class T> \
XVColorBase<T>::operator IM_TYPE() const { \
\
  IM_TYPE newIM(0,0); \
  if(newIM.ImageType() == this->ImageType()){ \
    newIM = dynamic_cast<IM_TYPE &>(const_cast<XVColorBase<T> & >(*this)); \
    return newIM; \
  } \
  newIM.resize(this->Width(), this->Height()); \
  XVImageIterator<T> fromIter(*this); \
  XVImageWIterator<IM_TYPE::PIXELTYPE>  toIter(newIM); \
\
  for(; !toIter.end(); ++toIter, ++fromIter){ \
    *toIter << *fromIter; \
  } \
  return newIM; \
};

CONVERT_OP_IMPL(XVImageRGB<XV_RGB15>);  
CONVERT_OP_IMPL(XVImageRGB<XV_RGB16>);
CONVERT_OP_IMPL(XVImageRGB<XV_RGB24>);  
CONVERT_OP_IMPL(XVImageRGB<XV_TRGB24>);  
CONVERT_OP_IMPL(XVImageRGB<XV_RGBA32>);
CONVERT_OP_IMPL(XVImageRGB<XV_GLRGBA32>);
CONVERT_OP_IMPL(XVImageYUV<XV_YUV24>);  
CONVERT_OP_IMPL(XVImageHSV<XV_HSV24>);

CONVERT_OP_IMPL(XVImageScalar<u_char>);
CONVERT_OP_IMPL(XVImageScalar<char>);
CONVERT_OP_IMPL(XVImageScalar<u_short>);
CONVERT_OP_IMPL(XVImageScalar<short>);
CONVERT_OP_IMPL(XVImageScalar<u_int>);
CONVERT_OP_IMPL(XVImageScalar<int>);
CONVERT_OP_IMPL(XVImageScalar<float>);
CONVERT_OP_IMPL(XVImageScalar<double>);

#include <XVImageYUV422.h>

template <class T>
XVColorBase<T>::operator XVImageYUV422() const {

  XVImageYUV422 newIM(0, 0);
  if(newIM.ImageType() == this->ImageType()){
    newIM = dynamic_cast<XVImageYUV422 & >(const_cast<XVColorBase<T> & >(*this)); 
    return newIM;
  }
  XVSize oldSize = (XVSize)(*this);
  (const_cast<XVColorBase<T> *>(this))->resize(2 * ((int)(this->Width() / 2)), this->Height());
  newIM.resize((int)(this->Width() / 2), this->Height());
  XVImageIterator<T> fromIter(*this);
  XVImageWIterator<XV_YUV422> toIter(newIM);
  T doublePix[2];
  for(; !toIter.end(); ++toIter, ++fromIter){
    doublePix[0] = *fromIter;
    ++fromIter;
    doublePix[1] = *fromIter;
    *toIter << doublePix;
  }
  (const_cast<XVColorBase<T> *>(this))->resize(oldSize);
  return newIM;
};
  
template class XVColorBase<XV_RGB15>;
template class XVColorBase<XV_RGB16>;
template class XVColorBase<XV_RGB24>;
template class XVColorBase<XV_TRGB24>;
template class XVColorBase<XV_RGBA32>;
template class XVColorBase<XV_GLRGBA32>;
template class XVColorBase<XV_YUV24>;
template class XVColorBase<XV_HSV24>;
